import React, { useEffect, useState } from 'react';
import { isEmpty, toUpper, startsWith, length } from 'ramda';
import { getAllCharacters } from './actions';
import Paper from '../../components/Paper';
import Character from './components/Character';
import './characters.scss';


const Characters = () => {

    /**
     *  Hook, který drží pole všech postav. K uložení hodnot dochází během renderu komponenty a s hodnotami se pak
     *  následně pracuje dále
     **/
    const [characters, setCharacters] = useState([]);

    /**
     * Hook, který drží hledavý výraz v inputu
     **/
    const [searchedExpression, setSearchedExpression] = useState('');

    /**
     *  Hook, zajišťující dotažení všech characterů a následně nastavení do state hooku komponenty.
     *  Podmínky if a funkci isEmpty zajistí, že k dotažení dojde pouze pokud je state prázdný
     **/
    useEffect(() => {
        if (isEmpty(characters)) {
            getAllCharacters().then((res) => setCharacters(res.data.docs));
        }
    });

    /**
     *  Funkce, která porovnává jestli hledaný výraz odpovídá jménu characteru, pokud ano, vrátí celý character.
     **/
    const checkCharacterName = (character) => {
        const characterName = toUpper(character.name);

        if (startsWith(toUpper(searchedExpression), characterName)) {
            return character;
        }
    };

    /**
     *  Funkce, která filtruje všechny charactery podle hledaného výrazu a vrací pole všech characterů, které
     *  odpovídají hledání.
     **/
    const filterCharactersBySearchedName = () => {
        const filteredCharacters = [];

        for (let i = 0; i < length(characters); i++) {

            const checkedCharacter = checkCharacterName(characters[i]);

            if (checkedCharacter) {
                filteredCharacters.push(checkedCharacter);
            }
        }

        return filteredCharacters;
    };

    /**
     *  Funkce, která mapuje všechny odpovídající charactery do komponenty <Character>. V případě, že uživatel
     *  ještě nic nezadal, vrací <h2> tag.
     **/
    const renderCharacters = () => {
        return (
            !searchedExpression ? <h2>Start typing...</h2> : filterCharactersBySearchedName().map((character) => (
                <Character key={character._id} character={character}/>
            ))
        );
    };

    return (
        <Paper>
            <h1>Characters</h1>
            <p>
                Na této stránce si ukážeme, jakým způsobem je možné filtrovat velká pole a hledat konkrétní hodnoty. Z
                beckendu dotáhneme pole o cca 1000 prvcích a následně pomocí search baru zadáme jméno postavy, kterou
                chceme najít. Vyhledávání funguje na principu našeptávače, kdy se nám začnou vracet položky na základě
                zadání hledaného řetězce. Na vyhledávání je aplikována funckionalita, která zajistí nerozlišování
                velkých a malých písmen, tedy "NOT KEY SENSITIVE".
            </p>

            <div className="search-bar">
                <label className="search-bar__label">Search character by name:</label>
                <input
                    className="search-bar__input"
                    type={'text'}
                    value={searchedExpression}
                    onChange={(event) => setSearchedExpression(event.target.value)}
                />
            </div>

            <div className="characters">
                {renderCharacters()}
            </div>
        </Paper>
    );
};

export default Characters;
